package com.ruoyi.web.controller.common;

import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson2.JSONObject;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.vod.model.v20170321.CreateUploadVideoRequest;
import com.aliyuncs.vod.model.v20170321.CreateUploadVideoResponse;
import com.aliyuncs.vod.model.v20170321.GetPlayInfoResponse;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.utils.vod.CreateVodDto;
import com.ruoyi.common.utils.vod.VodPlayAuthDto;
import com.ruoyi.common.utils.vod.VodUtils;
import com.ruoyi.common.utils.vod.VodZlsjsUtils;
import com.ruoyi.web.service.CommonService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.file.FileUploadUtils;
import com.ruoyi.common.utils.file.FileUtils;
import com.ruoyi.framework.config.ServerConfig;

/**
 * 通用请求处理
 * 
 * @author voluntary
 */
@RestController
@RequestMapping("/common")
@Api(tags = "app 前端 通用接口")
public class CommonController
{
    private static final Logger log = LoggerFactory.getLogger(CommonController.class);

    @Autowired
    private ServerConfig serverConfig;

    @Autowired
    private VodUtils vodUtils;
    @Autowired
    private VodZlsjsUtils vodZlsjsUtils;
    @Autowired
    private CommonService commonService;

    private static final String FILE_DELIMETER = ",";

    /**
     * 通用下载请求
     * 
     * @param fileName 文件名称
     * @param delete 是否删除
     */
    @GetMapping("/download")
    public void fileDownload(String fileName, Boolean delete, HttpServletResponse response, HttpServletRequest request)
    {
        try
        {
            if (!FileUtils.checkAllowDownload(fileName))
            {
                throw new Exception(StringUtils.format("文件名称({})非法，不允许下载。 ", fileName));
            }
            String realFileName = System.currentTimeMillis() + fileName.substring(fileName.indexOf("_") + 1);
            String filePath = RuoYiConfig.getDownloadPath() + fileName;

            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, realFileName);
            FileUtils.writeBytes(filePath, response.getOutputStream());
            if (delete)
            {
                FileUtils.deleteFile(filePath);
            }
        }
        catch (Exception e)
        {
            log.error("下载文件失败", e);
        }
    }

    /**
     * 通用上传请求（单个）
     */
    @PostMapping("/upload")
    public AjaxResult uploadFile(MultipartFile file) throws Exception
    {
        try
        {
            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            // 上传并返回新文件名称
            String fileName = FileUploadUtils.upload(filePath, file);
            String url = serverConfig.getUrl() + fileName;
            AjaxResult ajax = AjaxResult.success();
            ajax.put("url", url);
            ajax.put("fileName", fileName);
            ajax.put("newFileName", FileUtils.getName(fileName));
            ajax.put("originalFilename", file.getOriginalFilename());
            return ajax;
        }
        catch (Exception e)
        {
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 通用上传请求（多个）
     */
    @PostMapping("/uploads")
    public AjaxResult uploadFiles(List<MultipartFile> files) throws Exception
    {
        try
        {
            // 上传文件路径
            String filePath = RuoYiConfig.getUploadPath();
            List<String> urls = new ArrayList<String>();
            List<String> fileNames = new ArrayList<String>();
            List<String> newFileNames = new ArrayList<String>();
            List<String> originalFilenames = new ArrayList<String>();
            for (MultipartFile file : files)
            {
                // 上传并返回新文件名称
                String fileName = FileUploadUtils.upload(filePath, file);
                String url = serverConfig.getUrl() + fileName;
                urls.add(url);
                fileNames.add(fileName);
                newFileNames.add(FileUtils.getName(fileName));
                originalFilenames.add(file.getOriginalFilename());
            }
            AjaxResult ajax = AjaxResult.success();
            ajax.put("urls", StringUtils.join(urls, FILE_DELIMETER));
            ajax.put("fileNames", StringUtils.join(fileNames, FILE_DELIMETER));
            ajax.put("newFileNames", StringUtils.join(newFileNames, FILE_DELIMETER));
            ajax.put("originalFilenames", StringUtils.join(originalFilenames, FILE_DELIMETER));
            return ajax;
        }
        catch (Exception e)
        {
            return AjaxResult.error(e.getMessage());
        }
    }

    /**
     * 本地资源通用下载
     */
    @GetMapping("/download/resource")
    public void resourceDownload(String resource, HttpServletRequest request, HttpServletResponse response)
            throws Exception
    {
        try
        {
            if (!FileUtils.checkAllowDownload(resource))
            {
                throw new Exception(StringUtils.format("资源文件({})非法，不允许下载。 ", resource));
            }
            // 本地资源路径
            String localPath = RuoYiConfig.getProfile();
            // 数据库资源地址
            String downloadPath = localPath + StringUtils.substringAfter(resource, Constants.RESOURCE_PREFIX);
            // 下载名称
            String downloadName = StringUtils.substringAfterLast(downloadPath, "/");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, downloadName);
            FileUtils.writeBytes(downloadPath, response.getOutputStream());
        }
        catch (Exception e)
        {
            log.error("下载文件失败", e);
        }
    }

    @PostMapping("/uploadFile2OSS/{fileUploadType}")
    @ApiOperation("上传文件到阿里OSS")
    @ResponseBody
    public AjaxResult uploadFile2OSS(MultipartFile file, @PathVariable("fileUploadType") Integer fileUploadType){
        String result = commonService.uploadFile2OSS2(file,fileUploadType);

        return AjaxResult.success(result);
    }

    @RequestMapping(value = "/vod/createUploadVideo", method = {RequestMethod.GET,RequestMethod.POST})
    @ApiOperation("上传文件到阿里视频点播")
    public AjaxResult uploadVideo(String fileName, String title, Long fileSize) throws Exception {
        //上传文件修改为智隆叁加叁的vod
        DefaultAcsClient vodClient  = vodUtils.vodClient();

        CreateUploadVideoRequest createUploadVideoRequest = new CreateUploadVideoRequest();
        createUploadVideoRequest.setFileName(fileName);
        createUploadVideoRequest.setTitle(title);
        createUploadVideoRequest.setCateId(null);
        createUploadVideoRequest.setFileSize(fileSize);
        CreateUploadVideoResponse response = vodClient.getAcsResponse(createUploadVideoRequest);
        String uploadAddress = response.getUploadAddress();
        String uploadAuth = response.getUploadAuth();
        String videoId = response.getVideoId();

        CreateVodDto createVodDto = new CreateVodDto();
        createVodDto.setVideoId(videoId);
        createVodDto.setUploadAddress(uploadAddress);
        createVodDto.setUploadAuth(uploadAuth);

        return AjaxResult.success(createVodDto);
    }

    @GetMapping("/vod/getVideoPlayAuth")
    @ApiOperation("获取视频点播授权")
    public AjaxResult getVideoPlayAuth(String videoId) throws Exception {
        VodPlayAuthDto dto = new VodPlayAuthDto();
        try {
            dto = vodUtils.getVideoPlayAuth(videoId);
        }catch (Exception e){
            log.error(e.toString());
        }
        return AjaxResult.success(dto);
    }

    @GetMapping("/vod/getVideoPlayInfo")
    @ApiOperation("获取视频播放信息")
    public AjaxResult getVideoPlayInfo(String videoId) throws Exception {
        GetPlayInfoResponse response = new GetPlayInfoResponse();
        //智隆叁加叁的vod为 “zlsjs,” 开头
        Boolean isZlsjs=true;
        try {
            if(videoId.startsWith(Constants.ZLSJS_PREFIX)||Convert.toStrArray(videoId).length==2){
                videoId = Convert.toStrArray(videoId)[1];
                response = vodZlsjsUtils.getPlayInfo(videoId);
            }else {
                isZlsjs=false;
                response = vodUtils.getPlayInfo(videoId);
            }
        }catch (Exception e){
            if(isZlsjs){
                response = vodUtils.getPlayInfo(videoId);
            }else {
                response = vodZlsjsUtils.getPlayInfo(videoId);
            }
        }

        if(StringUtils.isNotNull(response)&& !CollectionUtils.isEmpty(response.getPlayInfoList())){
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("videoId",videoId);
            jsonObject.put("palyUrl",response.getPlayInfoList().get(0).getPlayURL());
            jsonObject.put("duration",response.getPlayInfoList().get(0).getDuration());
            jsonObject.put("coverUrl",StringUtils.EMPTY);
            if(StringUtils.isNotNull(response.getVideoBase())){
                jsonObject.put("coverUrl",response.getVideoBase().getCoverURL());
                jsonObject.put("title",response.getVideoBase().getTitle());
                jsonObject.put("videoId",response.getVideoBase().getVideoId());
            }

            return AjaxResult.success(jsonObject);
        }
        return AjaxResult.error("未找到有效的url");
    }
}
