package com.ruoyi.web.controller.ie;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ie.domain.AEnrollSpecial;
import com.ruoyi.ie.service.IAEnrollSpecialService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 院校考生专项考核Controller
 * 
 * @author voluntary
 * @date 2023-11-26
 */
@RestController
@RequestMapping("/ie/aEnrollSpecial")
public class AEnrollSpecialController extends BaseController
{
    @Autowired
    private IAEnrollSpecialService aEnrollSpecialService;

    /**
     * 查询院校考生专项考核列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollSpecial:list')")
    @GetMapping("/list")
    public TableDataInfo list(AEnrollSpecial aEnrollSpecial)
    {
        startPage();
        List<AEnrollSpecial> list = aEnrollSpecialService.selectAEnrollSpecialList(aEnrollSpecial);
        return getDataTable(list);
    }

    /**
     * 导出院校考生专项考核列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollSpecial:export')")
    @Log(title = "院校考生专项考核", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AEnrollSpecial aEnrollSpecial)
    {
        List<AEnrollSpecial> list = aEnrollSpecialService.selectAEnrollSpecialList(aEnrollSpecial);
        ExcelUtil<AEnrollSpecial> util = new ExcelUtil<AEnrollSpecial>(AEnrollSpecial.class);
        util.exportExcel(response, list, "院校考生专项考核数据");
    }

    /**
     * 获取院校考生专项考核详细信息
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollSpecial:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(aEnrollSpecialService.selectAEnrollSpecialById(id));
    }

    /**
     * 新增院校考生专项考核
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollSpecial:add')")
    @Log(title = "院校考生专项考核", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AEnrollSpecial aEnrollSpecial)
    {
        return toAjax(aEnrollSpecialService.insertAEnrollSpecial(aEnrollSpecial));
    }

    /**
     * 修改院校考生专项考核
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollSpecial:edit')")
    @Log(title = "院校考生专项考核", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AEnrollSpecial aEnrollSpecial)
    {
        return toAjax(aEnrollSpecialService.updateAEnrollSpecial(aEnrollSpecial));
    }

    /**
     * 删除院校考生专项考核
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollSpecial:remove')")
    @Log(title = "院校考生专项考核", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(aEnrollSpecialService.deleteAEnrollSpecialByIds(ids));
    }
}
