package com.ruoyi.ie.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ie.domain.AEnrollUniversity;
import com.ruoyi.ie.service.IAEnrollUniversityService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 院校考生考核方式总体Controller
 * 
 * @author voluntary
 * @date 2023-12-06
 */
@RestController
@RequestMapping("/ie/aEnrollUniversity")
public class AEnrollUniversityController extends BaseController
{
    @Autowired
    private IAEnrollUniversityService aEnrollUniversityService;

    /**
     * 查询院校考生考核方式总体列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollUniversity:list')")
    @GetMapping("/list")
    public TableDataInfo list(AEnrollUniversity aEnrollUniversity)
    {
        startPage();
        List<AEnrollUniversity> list = aEnrollUniversityService.selectAEnrollUniversityList(aEnrollUniversity);
        return getDataTable(list);
    }

    /**
     * 导出院校考生考核方式总体列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollUniversity:export')")
    @Log(title = "院校考生考核方式总体", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AEnrollUniversity aEnrollUniversity)
    {
        List<AEnrollUniversity> list = aEnrollUniversityService.selectAEnrollUniversityList(aEnrollUniversity);
        ExcelUtil<AEnrollUniversity> util = new ExcelUtil<AEnrollUniversity>(AEnrollUniversity.class);
        util.exportExcel(response, list, "院校考生考核方式总体数据");
    }

    /**
     * 获取院校考生考核方式总体详细信息
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollUniversity:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(aEnrollUniversityService.selectAEnrollUniversityById(id));
    }

    /**
     * 新增院校考生考核方式总体
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollUniversity:add')")
    @Log(title = "院校考生考核方式总体", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AEnrollUniversity aEnrollUniversity)
    {
        return toAjax(aEnrollUniversityService.insertAEnrollUniversity(aEnrollUniversity));
    }

    /**
     * 修改院校考生考核方式总体
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollUniversity:edit')")
    @Log(title = "院校考生考核方式总体", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AEnrollUniversity aEnrollUniversity)
    {
        return toAjax(aEnrollUniversityService.updateAEnrollUniversity(aEnrollUniversity));
    }

    /**
     * 删除院校考生考核方式总体
     */
    @PreAuthorize("@ss.hasPermi('ie:aEnrollUniversity:remove')")
    @Log(title = "院校考生考核方式总体", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(aEnrollUniversityService.deleteAEnrollUniversityByIds(ids));
    }
}
