package com.ruoyi.web.controller.ie;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ie.domain.AMarjorEnroll;
import com.ruoyi.ie.service.IAMarjorEnrollService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 专业录取情况Controller
 * 
 * @author voluntary
 * @date 2023-11-26
 */
@RestController
@RequestMapping("/ie/aMarjorEnroll")
public class AMarjorEnrollController extends BaseController
{
    @Autowired
    private IAMarjorEnrollService aMarjorEnrollService;

    /**
     * 查询专业录取情况列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorEnroll:list')")
    @GetMapping("/list")
    public TableDataInfo list(AMarjorEnroll aMarjorEnroll)
    {
        startPage();
        List<AMarjorEnroll> list = aMarjorEnrollService.selectAMarjorEnrollList(aMarjorEnroll);
        return getDataTable(list);
    }

    /**
     * 导出专业录取情况列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorEnroll:export')")
    @Log(title = "专业录取情况", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AMarjorEnroll aMarjorEnroll)
    {
        List<AMarjorEnroll> list = aMarjorEnrollService.selectAMarjorEnrollList(aMarjorEnroll);
        ExcelUtil<AMarjorEnroll> util = new ExcelUtil<AMarjorEnroll>(AMarjorEnroll.class);
        util.exportExcel(response, list, "专业录取情况数据");
    }

    /**
     * 获取专业录取情况详细信息
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorEnroll:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(aMarjorEnrollService.selectAMarjorEnrollById(id));
    }

    /**
     * 新增专业录取情况
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorEnroll:add')")
    @Log(title = "专业录取情况", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AMarjorEnroll aMarjorEnroll)
    {
        return toAjax(aMarjorEnrollService.insertAMarjorEnroll(aMarjorEnroll));
    }

    /**
     * 修改专业录取情况
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorEnroll:edit')")
    @Log(title = "专业录取情况", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AMarjorEnroll aMarjorEnroll)
    {
        return toAjax(aMarjorEnrollService.updateAMarjorEnroll(aMarjorEnroll));
    }

    /**
     * 删除专业录取情况
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorEnroll:remove')")
    @Log(title = "专业录取情况", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(aMarjorEnrollService.deleteAMarjorEnrollByIds(ids));
    }
}
