package com.ruoyi.web.controller.ie;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ie.domain.AMarjorPlan;
import com.ruoyi.ie.service.IAMarjorPlanService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 专业计划要求Controller
 * 
 * @author voluntary
 * @date 2023-11-26
 */
@RestController
@RequestMapping("/ie/aMarjorPlan")
public class AMarjorPlanController extends BaseController
{
    @Autowired
    private IAMarjorPlanService aMarjorPlanService;

    /**
     * 查询专业计划要求列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorPlan:list')")
    @GetMapping("/list")
    public TableDataInfo list(AMarjorPlan aMarjorPlan)
    {
        startPage();
        List<AMarjorPlan> list = aMarjorPlanService.selectAMarjorPlanList(aMarjorPlan);
        return getDataTable(list);
    }

    /**
     * 导出专业计划要求列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorPlan:export')")
    @Log(title = "专业计划要求", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AMarjorPlan aMarjorPlan)
    {
        List<AMarjorPlan> list = aMarjorPlanService.selectAMarjorPlanList(aMarjorPlan);
        ExcelUtil<AMarjorPlan> util = new ExcelUtil<AMarjorPlan>(AMarjorPlan.class);
        util.exportExcel(response, list, "专业计划要求数据");
    }

    /**
     * 获取专业计划要求详细信息
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorPlan:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(aMarjorPlanService.selectAMarjorPlanById(id));
    }

    /**
     * 新增专业计划要求
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorPlan:add')")
    @Log(title = "专业计划要求", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AMarjorPlan aMarjorPlan)
    {
        return toAjax(aMarjorPlanService.insertAMarjorPlan(aMarjorPlan));
    }

    /**
     * 修改专业计划要求
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorPlan:edit')")
    @Log(title = "专业计划要求", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AMarjorPlan aMarjorPlan)
    {
        return toAjax(aMarjorPlanService.updateAMarjorPlan(aMarjorPlan));
    }

    /**
     * 删除专业计划要求
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorPlan:remove')")
    @Log(title = "专业计划要求", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(aMarjorPlanService.deleteAMarjorPlanByIds(ids));
    }
}
