package com.ruoyi.web.controller.ie;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ie.domain.AMarjorSubmit;
import com.ruoyi.ie.service.IAMarjorSubmitService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 专业报名划线Controller
 * 
 * @author voluntary
 * @date 2023-11-26
 */
@RestController
@RequestMapping("/ie/aMarjorSubmit")
public class AMarjorSubmitController extends BaseController
{
    @Autowired
    private IAMarjorSubmitService aMarjorSubmitService;

    /**
     * 查询专业报名划线列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorSubmit:list')")
    @GetMapping("/list")
    public TableDataInfo list(AMarjorSubmit aMarjorSubmit)
    {
        startPage();
        List<AMarjorSubmit> list = aMarjorSubmitService.selectAMarjorSubmitList(aMarjorSubmit);
        return getDataTable(list);
    }

    /**
     * 导出专业报名划线列表
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorSubmit:export')")
    @Log(title = "专业报名划线", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, AMarjorSubmit aMarjorSubmit)
    {
        List<AMarjorSubmit> list = aMarjorSubmitService.selectAMarjorSubmitList(aMarjorSubmit);
        ExcelUtil<AMarjorSubmit> util = new ExcelUtil<AMarjorSubmit>(AMarjorSubmit.class);
        util.exportExcel(response, list, "专业报名划线数据");
    }

    /**
     * 获取专业报名划线详细信息
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorSubmit:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(aMarjorSubmitService.selectAMarjorSubmitById(id));
    }

    /**
     * 新增专业报名划线
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorSubmit:add')")
    @Log(title = "专业报名划线", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody AMarjorSubmit aMarjorSubmit)
    {
        return toAjax(aMarjorSubmitService.insertAMarjorSubmit(aMarjorSubmit));
    }

    /**
     * 修改专业报名划线
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorSubmit:edit')")
    @Log(title = "专业报名划线", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody AMarjorSubmit aMarjorSubmit)
    {
        return toAjax(aMarjorSubmitService.updateAMarjorSubmit(aMarjorSubmit));
    }

    /**
     * 删除专业报名划线
     */
    @PreAuthorize("@ss.hasPermi('ie:aMarjorSubmit:remove')")
    @Log(title = "专业报名划线", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(aMarjorSubmitService.deleteAMarjorSubmitByIds(ids));
    }
}
