package com.ruoyi.web.controller.system;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.SmsBlacklist;
import com.ruoyi.system.service.ISmsBlacklistService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 手机黑名单Controller
 *
 * @author ie
 * @date 2025-01-25
 */
@RestController
@RequestMapping("/system/smsBlacklist")
public class SmsBlacklistController extends BaseController
{
    @Autowired
    private ISmsBlacklistService smsBlacklistService;

    /**
     * 查询手机黑名单列表
     */
    @PreAuthorize("@ss.hasPermi('system:smsBlacklist:list')")
    @GetMapping("/list")
    public TableDataInfo list(SmsBlacklist smsBlacklist)
    {
        startPage();
        List<SmsBlacklist> list = smsBlacklistService.selectSmsBlacklistList(smsBlacklist);
        return getDataTable(list);
    }

    /**
     * 导出手机黑名单列表
     */
    @PreAuthorize("@ss.hasPermi('system:smsBlacklist:export')")
    @Log(title = "手机黑名单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, SmsBlacklist smsBlacklist)
    {
        List<SmsBlacklist> list = smsBlacklistService.selectSmsBlacklistList(smsBlacklist);
        ExcelUtil<SmsBlacklist> util = new ExcelUtil<SmsBlacklist>(SmsBlacklist.class);
        util.exportExcel(response, list, "手机黑名单数据");
    }

    /**
     * 获取手机黑名单详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:smsBlacklist:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(smsBlacklistService.selectSmsBlacklistById(id));
    }

    /**
     * 新增手机黑名单
     */
    @PreAuthorize("@ss.hasPermi('system:smsBlacklist:add')")
    @Log(title = "手机黑名单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody SmsBlacklist smsBlacklist)
    {
        return toAjax(smsBlacklistService.insertSmsBlacklist(smsBlacklist));
    }

    /**
     * 修改手机黑名单
     */
    @PreAuthorize("@ss.hasPermi('system:smsBlacklist:edit')")
    @Log(title = "手机黑名单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody SmsBlacklist smsBlacklist)
    {
        return toAjax(smsBlacklistService.updateSmsBlacklist(smsBlacklist));
    }

    /**
     * 删除手机黑名单
     */
    @PreAuthorize("@ss.hasPermi('system:smsBlacklist:remove')")
    @Log(title = "手机黑名单", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(smsBlacklistService.deleteSmsBlacklistByIds(ids));
    }
}
