package com.ruoyi.web.controller.system;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.uuid.UUID;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.service.IOssService;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.system.service.ShortMessageService;
import com.ruoyi.voluntary.domain.BusiCards;
import com.ruoyi.voluntary.service.IBusiCardsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Date;
import java.util.List;

/**
 * 个人信息 业务处理
 *
 * @author voluntary
 */
@RestController
@RequestMapping("/system/user/profile")
public class SysProfileController extends BaseController
{
    @Autowired
    private ISysUserService userService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private IBusiCardsService cardsService;

    @Autowired
    private IOssService ossService;

    @Autowired
    private ShortMessageService shortMessageService;
    /**
     * 个人信息
     */
    @GetMapping
    public AjaxResult profile()
    {
        LoginUser loginUser = getLoginUser();
        SysUser user = loginUser.getUser();
        AjaxResult ajax = AjaxResult.success(user);
        ajax.put("roleGroup", userService.selectUserRoleGroup(loginUser.getUsername()));
        ajax.put("postGroup", userService.selectUserPostGroup(loginUser.getUsername()));
        return ajax;
    }

    /**
     * 修改用户
     */
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult updateProfile(@RequestBody SysUser user)
    {
        LoginUser loginUser = getLoginUser();
        SysUser sysUser = loginUser.getUser();
        user.setUserName(sysUser.getUserName());
        if (StringUtils.isNotEmpty(user.getPhonenumber())
                && UserConstants.NOT_UNIQUE.equals(userService.checkPhoneUnique(user)))
        {
            return error("修改用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        if (StringUtils.isNotEmpty(user.getEmail())
                && UserConstants.NOT_UNIQUE.equals(userService.checkEmailUnique(user)))
        {
            return error("修改用户'" + user.getUserName() + "'失败，邮箱账号已存在");
        }
        user.setUserId(sysUser.getUserId());
        user.setPassword(null);
        user.setAvatar(null);
        user.setDeptId(null);
        if (userService.updateUserProfile(user) > 0)
        {
            // 更新缓存用户信息
            sysUser.setNickName(user.getNickName());
            sysUser.setPhonenumber(user.getPhonenumber());
            sysUser.setEmail(user.getEmail());
            sysUser.setSex(user.getSex());
            tokenService.setLoginUser(loginUser);
            return success();
        }
        return error("修改个人信息异常，请联系管理员");
    }

    /**
     * 重置密码
     */
    @Log(title = "个人信息", businessType = BusinessType.UPDATE)
//    @PutMapping("/updatePwd")
    @RequestMapping(value = "updatePwd", method = {RequestMethod.PUT, RequestMethod.GET, RequestMethod.POST})
    public AjaxResult updatePwd(String oldPassword, String newPassword)
    {
        LoginUser loginUser = getLoginUser();
        String userName = loginUser.getUsername();
        String password = loginUser.getPassword();
        if (!SecurityUtils.matchesPassword(oldPassword, password))
        {
            return error("修改密码失败，旧密码错误");
        }
        if (SecurityUtils.matchesPassword(newPassword, password))
        {
            return error("新密码不能与旧密码相同");
        }
        SysUser pwdUser = new SysUser();
        pwdUser.setUserId(loginUser.getUser().getUserId());
        pwdUser.setPassword(SecurityUtils.encryptPassword(newPassword));
        pwdUser.setPassword2(newPassword);
        int count = userService.updateUserStatus(pwdUser) ;
        if (count>0)
        {
            //更新card中passoword
            BusiCards card = cardsService.selectBusiCardsByCardNo(userName);
            if(null!=card){
                card.setPassword(newPassword);
                card.setUpdateTime(new Date());
                cardsService.updateBusiCards(card);
            }
            // 更新缓存用户密码
            loginUser.getUser().setPassword(SecurityUtils.encryptPassword(newPassword));
            loginUser.getUser().setPassword2(newPassword);
            tokenService.setLoginUser(loginUser);
            return success();
        }
        return error("修改密码异常，请联系管理员");
    }

    /**
     * 头像上传
     */
    @Log(title = "用户头像", businessType = BusinessType.UPDATE)
    @PostMapping("/avatar")
    public AjaxResult avatar(@RequestParam("avatarfile") MultipartFile file) throws Exception
    {
        if (!file.isEmpty())
        {
            LoginUser loginUser = getLoginUser();
            // String avatar = FileUploadUtils.upload(RuoYiConfig.getAvatarPath(), file, MimeTypeUtils.IMAGE_EXTENSION);
            String avatar = ossService.upload(file.getInputStream(), loginUser.getUser().getUserName()+"_"+ UUID.randomUUID().toString() + ".png", "voluntary/avatar/");
            if (userService.updateUserAvatar(loginUser.getUsername(), avatar))
            {
                AjaxResult ajax = AjaxResult.success();
                ajax.put("imgUrl", avatar);
                // 更新缓存用户头像
                loginUser.getUser().setAvatar(avatar);
                tokenService.setLoginUser(loginUser);
                return ajax;
            }
        }
        return error("上传图片异常，请联系管理员");
    }

    @Log(title = "更新手机号", businessType = BusinessType.UPDATE)
    @GetMapping("/updatePhonenumber")
    public AjaxResult updatePhonenumber(String code,String phonenumber) throws IOException
    {
        if(StringUtils.isBlank(phonenumber)){
            return AjaxResult.error("手机号不能为空");
        }
        List<SysUser> userList=userService.selectUserByPhonenumber(phonenumber);
        if(!CollectionUtils.isEmpty(userList)){
            return AjaxResult.error("手机号已存在，请更换后重新绑定！");
        }

        boolean isSuccess = shortMessageService.checkCode(phonenumber, code);
        if(!isSuccess){
            return AjaxResult.error("验证码无效");
        }

        LoginUser loginUser=SecurityUtils.getLoginUser();
        //更新sys_user手机号
        SysUser user = userService.selectUserByUserName(loginUser.getUsername());
        user.setPhonenumber(phonenumber);
        Date now = new Date();
        user.setUpdateTime(now);
        userService.updateUserStatus(user);

        // 更新缓存
        loginUser.getUser().setPhonenumber(phonenumber);
        tokenService.setLoginUser(loginUser);

        return AjaxResult.success("修改手机号成功");
    }
}
