package com.ruoyi.system.service.impl;

import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.enums.DataSourceType;
import com.ruoyi.system.domain.ZuserToken;
import com.ruoyi.system.mapper.ZuserTokenMapper;
import com.ruoyi.system.service.IZuserTokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * tokenService业务层处理
 *
 * @author mingxue
 * @date 2021-06-15
 */
@Service
@DataSource(DataSourceType.MASTER)
public class ZuserTokenServiceImpl implements IZuserTokenService
{
    @Autowired
    private ZuserTokenMapper zuserTokenMapper;

    /**
     * 查询token
     *
     * @param id tokenID
     * @return token
     */
    @Override
    public ZuserToken selectZuserTokenById(Integer id)
    {
        return zuserTokenMapper.selectZuserTokenById(id);
    }

    @Override
    public ZuserToken selectZuserTokenByUserId(Long userId) {
        return zuserTokenMapper.selectZuserTokenByUserId(userId);
    }

    /**
     * 查询token列表
     *
     * @param zuserToken token
     * @return token
     */
    @Override
    public List<ZuserToken> selectZuserTokenList(ZuserToken zuserToken)
    {
        return zuserTokenMapper.selectZuserTokenList(zuserToken);
    }

    @Override
    public List<ZuserToken> selectTopZuserTokenList(ZuserToken zuserToken) {
        return zuserTokenMapper.selectTopZuserTokenList(zuserToken);
    }

    /**
     * 新增token
     *
     * @param zuserToken token
     * @return 结果
     */
    @Override
    public int insertZuserToken(ZuserToken zuserToken)
    {
        return zuserTokenMapper.insertZuserToken(zuserToken);
    }

    /**
     * 修改token
     *
     * @param zuserToken token
     * @return 结果
     */
    @Override
    public int updateZuserToken(ZuserToken zuserToken)
    {
        return zuserTokenMapper.updateZuserToken(zuserToken);
    }

    /**
     * 批量删除token
     *
     * @param ids 需要删除的tokenID
     * @return 结果
     */
    @Override
    public int deleteZuserTokenByIds(Long[] ids)
    {
        return zuserTokenMapper.deleteZuserTokenByIds(ids);
    }

    /**
     * 删除token信息
     *
     * @param id tokenID
     * @return 结果
     */
    @Override
    public int deleteZuserTokenById(Long id)
    {
        return zuserTokenMapper.deleteZuserTokenById(id);
    }
}
